@@ -3,7 +3,7 @@ from __future__ import print_function, division, absolute_import
 from llvmlite.llvmpy.core import Type, Builder, Constant
 import llvmlite.llvmpy.core as lc
 
-from numba import types, cgutils
+from numba import types, cgutils, config
 
 
 class _ArgManager(object):
@@ -142,11 +142,13 @@ class PyCallWrapper(object):
         # Extract the Environment object from the Closure
         envptr, env_manager = self.get_env(api, builder, closure)
 
-        status, res = self.context.call_conv.call_function(
+        status, retval = self.context.call_conv.call_function(
             builder, self.func, self.fndesc.restype, self.fndesc.argtypes,
             innerargs, envptr)
         # Do clean up
+        self.debug_print(builder, "# callwrapper: emit_cleanup")
         cleanup_manager.emit_cleanup()
+        self.debug_print(builder, "# callwrapper: emit_cleanup end")
 
         # Determine return status
         with cgutils.if_likely(builder, status.is_ok):
@@ -154,9 +156,9 @@ class PyCallWrapper(object):
             with builder.if_then(status.is_none):
                 api.return_none()
 
-            retval = api.from_native_return(res, self._simplified_return_type(),
-                                            env_manager)
-            builder.ret(retval)
+            retty = self._simplified_return_type()
+            obj = api.from_native_return(retval, retty, env_manager)
+            builder.ret(obj)
 
         with builder.if_then(builder.not_(status.is_python_exc)):
             # User exception raised
@@ -211,3 +213,7 @@ class PyCallWrapper(object):
             return restype.type
         else:
             return restype
+
+    def debug_print(self, builder, msg):
+        if config.DEBUG_JIT:
+            self.context.debug_print(builder, "DEBUGJIT: {0}".format(msg))
