@@ -63,7 +63,7 @@ def to_device(obj, stream=0, copy=True, to=None):
 def device_array(shape, dtype=np.float, strides=None, order='C', stream=0):
     """device_array(shape, dtype=np.float, strides=None, order='C', stream=0)
 
-    Allocate an empty device ndarray. Similar to numpy.empty()
+    Allocate an empty device ndarray. Similar to :meth:`numpy.empty`.
     """
     shape, strides, dtype = _prepare_shape_strides_dtype(shape, strides, dtype,
                                                          order)
@@ -113,7 +113,7 @@ def mapped_array(shape, dtype=np.float, strides=None, order='C', stream=0,
 
 
 def synchronize():
-    "Synchronize current context"
+    "Synchronize the current context."
     return current_context().synchronize()
 
 
@@ -202,19 +202,22 @@ def mapped(*arylist, **kws):
 
 
 def event(timing=True):
-    """Create a CUDA event.
+    """
+    Create a CUDA event. Timing data is only recorded by the event if it is
+    created with ``timing=True``.
     """
     evt = current_context().create_event(timing=timing)
     return evt
 
+event_elapsed_time = driver.event_elapsed_time
+
 # Device selection
 
 def select_device(device_id):
-    """Creates a new CUDA context with the selected device.
-    The context is associated with the current thread.
-    Numba currently allows only one context per thread.
+    """
+    Make the context associated with device *device_id* the current context.
 
-    Returns a device instance
+    Returns a Device instance.
 
     Raises exception on error.
     """
@@ -228,14 +231,14 @@ def get_current_device():
 
 
 def list_devices():
-    "List all CUDA devices"
+    "Return a list of all detected devices"
     return devices.gpus
 
 
 def close():
-    """Explicitly closes the context.
-
-    Destroy the current context of the current thread
+    """
+    Explicitly clears all contexts in the current thread, and destroys all
+    contexts if the current thread is the main thread.
     """
     devices.reset()
 
@@ -245,7 +248,10 @@ def _auto_device(ary, stream=0, copy=True):
 
 
 def detect():
-    """Detect hardware support
+    """
+    Detect supported CUDA hardware and print a summary of the detected hardware.
+
+    Returns a boolean indicating whether any supported devices were detected.
     """
     devlist = list_devices()
     print('Found %d CUDA devices' % len(devlist))
@@ -278,12 +284,6 @@ def defer_cleanup():
         yield
 
 
-# TODO
-# Temporary entry-point to debug a failure for nvidia profiling tools to
-# record any kind of events.  Manually invocation of _profile_stop seems to be
-# necessary only on windows.
-# Should we make cuda.close() call _profile_stop()?
-_profiling = require_context(driver.profiling)
-_profile_start = require_context(driver.profile_start)
-_profile_stop = require_context(driver.profile_stop)
-
+profiling = require_context(driver.profiling)
+profile_start = require_context(driver.profile_start)
+profile_stop = require_context(driver.profile_stop)
