@@ -9,8 +9,8 @@ from numba import typing, types, cgutils
 from numba.utils import cached_property
 from numba.targets.base import BaseContext
 from numba.targets.callconv import MinimalCallConv
-from numba.targets import cmathimpl
-from numba.typing import cmathdecl
+from numba.targets import cmathimpl, operatorimpl
+from numba.typing import cmathdecl, operatordecl
 from numba.funcdesc import transform_arg_name
 from .cudadrv import nvvm
 from . import codegen, nvvmutils
@@ -27,6 +27,7 @@ class CUDATypingContext(typing.BaseContext):
         self.install(cudadecl.registry)
         self.install(cudamath.registry)
         self.install(cmathdecl.registry)
+        self.install(operatordecl.registry)
 
 # -----------------------------------------------------------------------------
 # Implementation
@@ -43,13 +44,15 @@ class CUDATargetContext(BaseContext):
         return self._internal_codegen._create_empty_module(name)
 
     def init(self):
-        from . import cudaimpl, libdevice
+        from . import cudaimpl, printimpl, libdevice
 
         self._internal_codegen = codegen.JITCUDACodegen("numba.cuda.jit")
 
         self.install_registry(cudaimpl.registry)
+        self.install_registry(printimpl.registry)
         self.install_registry(libdevice.registry)
         self.install_registry(cmathimpl.registry)
+        self.install_registry(operatorimpl.registry)
         self._target_data = ll.create_target_data(nvvm.default_data_layout)
 
     def jit_codegen(self):
