@@ -1,15 +1,27 @@
+# -*- coding: utf8 -*-
+
 from __future__ import print_function, division, absolute_import
 
+import contextlib
 import functools
+import errno
+import itertools
 import inspect
+import os
+from .six.moves import cPickle as pickle
+import struct
 import sys
+import warnings
 
+import numba
 from numba import _dispatcher, compiler, utils, types
 from numba.typeconv.rules import default_type_manager
 from numba import sigutils, serialize, types, typing
 from numba.typing.templates import fold_arguments
+from numba.typing.typeof import typeof
 from numba.bytecode import get_code_object
 from numba.six import create_bound_method, next
+from .config import NumbaWarning
 
 
 class _OverloadedBase(_dispatcher.Dispatcher):
@@ -230,7 +242,9 @@ class _OverloadedBase(_dispatcher.Dispatcher):
         This is called from numba._dispatcher as a fallback if the native code
         cannot decide the type.
         """
-        tp = self.typingctx.resolve_argument_type(val)
+        # Not going through the resolve_argument_type() indirection
+        # can shape a couple µs.
+        tp = typeof(val)
         if tp is None:
             tp = types.pyobject
         return tp
@@ -268,9 +282,13 @@ class Overloaded(_OverloadedBase):
 
         self.targetoptions = targetoptions
         self.locals = locals
+        self._cache = NullCache()
 
         self.typingctx.insert_overloaded(self)
 
+    def enable_caching(self):
+        self._cache = FunctionCache(self.py_func)
+
     def __get__(self, obj, objtype=None):
         '''Allow a JIT function to be bound as a method to an object'''
         if obj is None:  # Unbound method
@@ -308,11 +326,19 @@ class Overloaded(_OverloadedBase):
         with self._compile_lock:
             args, return_type = sigutils.normalize_signature(sig)
             # Don't recompile if signature already exists
-            # (e.g. if another thread compiled it before we got the lock)
             existing = self.overloads.get(tuple(args))
             if existing is not None:
                 return existing
 
+            cres = self._cache.load_overload(sig, self.targetctx)
+            if cres is not None:
+                # XXX fold this in add_overload()? (also see compiler.py)
+                if not cres.objectmode and not cres.interpmode:
+                    self.targetctx.insert_user_function(cres.entry_point,
+                                                   cres.fndesc, [cres.library])
+                self.add_overload(cres)
+                return cres.entry_point
+
             flags = compiler.Flags()
             self.targetdescr.options.parse_as_flags(flags, self.targetoptions)
 
@@ -326,17 +352,22 @@ class Overloaded(_OverloadedBase):
                 raise cres.typing_error
 
             self.add_overload(cres)
+            self._cache.save_overload(sig, cres)
             return cres.entry_point
 
     def recompile(self):
         """
         Recompile all signatures afresh.
         """
-        sigs = [cr.signature for cr in self._compileinfos.values()]
+        # A subtle point: self.overloads has argument type tuples, while
+        # while self._compileinfos has full signatures including return
+        # types.  This has an effect on cache lookups...
+        sigs = list(self.overloads)
         old_can_compile = self._can_compile
         # Ensure the old overloads are disposed of, including compiled functions.
         self._make_finalizer()()
         self._reset_overloads()
+        self._cache.flush()
         self._can_compile = True
         try:
             for sig in sigs:
@@ -399,5 +430,242 @@ class LiftedLoop(_OverloadedBase):
             return cres.entry_point
 
 
-# Initialize dispatcher
-_dispatcher.init_types(dict((str(t), t._code) for t in types.number_domain))
+# Initialize typeof machinery
+_dispatcher.typeof_init(dict((str(t), t._code) for t in types.number_domain))
+
+
+class NullCache(object):
+
+    def load_overload(self, sig, target_context):
+        pass
+
+    def save_overload(self, sig, cres):
+        pass
+
+    def enable(self):
+        pass
+
+    def disable(self):
+        pass
+
+    def flush(self):
+        pass
+
+
+class FunctionCache(object):
+    """
+    A per-function compilation cache.  The cache saves data in separate
+    data files and maintains information in an index file.
+
+    There is one index file per function and Python version
+    ("function_name-<lineno>.pyXY.nbi") which contains a mapping of
+    signatures and architectures to data files.
+    It is prefixed by a versioning key and a timestamp of the Python source
+    file containing the function.
+
+    There is one data file ("function_name-<lineno>.pyXY.<number>.nbc")
+    per function, function signature, target architecture and Python version.
+
+    Separate index and data files per Python version avoid pickle
+    compatibility problems.
+    """
+
+    _source_stamp = None
+
+    def __init__(self, py_func):
+        try:
+            qualname = py_func.__qualname__
+        except AttributeError:
+            qualname = py_func.__name__
+        # Keep the last dotted component, since the package name is already
+        # encoded in the directory.
+        modname = py_func.__module__.split('.')[-1]
+        self._funcname = qualname.split('.')[-1]
+        self._fullname = "%s.%s" % (modname, qualname)
+        self._source_path = inspect.getfile(py_func)
+        self._is_closure = bool(py_func.__closure__)
+        self._lineno = py_func.__code__.co_firstlineno
+        # NOTE: this assumes the __pycache__ directory is writable, which
+        # is false for system installs, but true for conda environments
+        # and local work directories.
+        self._cache_path = os.path.join(os.path.dirname(self._source_path),
+                                        '__pycache__')
+        abiflags = getattr(sys, 'abiflags', '')
+        # '<' and '>' can appear in the qualname (e.g. '<locals>') but
+        # are forbidden in Windows filenames
+        fixed_fullname = self._fullname.replace('<', '').replace('>', '')
+        filename_base = (
+            '%s-%d.py%d%d%s' % (fixed_fullname, self._lineno,
+                                sys.version_info[0], sys.version_info[1],
+                                abiflags)
+            )
+        self._index_name = '%s.nbi' % (filename_base,)
+        self._index_path = os.path.join(self._cache_path, self._index_name)
+        self._data_name_pattern = '%s.{number:d}.nbc' % (filename_base,)
+
+        self.enable()
+
+    def __repr__(self):
+        return "<%s fullname=%r>" % (self.__class__.__name__, self._fullname)
+
+    def enable(self):
+        self._enabled = True
+        st = os.stat(self._source_path)
+        # This may be a bit strict but avoids us maintaining a magic number
+        self._version = numba.__version__
+        # We use both timestamp and size as some filesystems only have second
+        # granularity.
+        self._source_stamp = st.st_mtime, st.st_size
+
+    def disable(self):
+        self._enabled = False
+
+    def flush(self):
+        self._save_index({})
+
+    def load_overload(self, sig, target_context):
+        """
+        Load and recreate the cached CompileResult for the given signature,
+        using the *target_context*.
+        """
+        if not self._enabled:
+            return
+        overloads = self._load_index()
+        key = self._index_key(sig, target_context.jit_codegen())
+        data_name = overloads.get(key)
+        if data_name is None:
+            return
+        try:
+            return self._load_data(data_name, target_context)
+        except EnvironmentError:
+            # File could have been removed while the index still refers it.
+            return
+
+    def save_overload(self, sig, cres):
+        """
+        Save the CompileResult for the given signature in the cache.
+        """
+        if not self._enabled:
+            return
+        if not self._check_cachable(cres):
+            return
+        try:
+            os.mkdir(self._cache_path)
+        except OSError as e:
+            if e.errno != errno.EEXIST:
+                raise
+        overloads = self._load_index()
+        key = self._index_key(sig, cres.library.codegen)
+        try:
+            # If key already exists, we will overwrite the file
+            data_name = overloads[key]
+        except KeyError:
+            # Find an available name for the data file
+            existing = set(overloads.values())
+            for i in itertools.count(1):
+                data_name = self._data_name(i)
+                if data_name not in existing:
+                    break
+            overloads[key] = data_name
+            self._save_index(overloads)
+
+        self._save_data(data_name, cres)
+
+    def _check_cachable(self, cres):
+        """
+        Check cachability of the given compile result.
+        """
+        cannot_cache = None
+        if self._is_closure:
+            cannot_cache = "as it uses outer variables in a closure"
+        elif cres.lifted:
+            cannot_cache = "as it uses lifted loops"
+        elif cres.has_dynamic_globals:
+            cannot_cache = "as it uses dynamic globals (such as ctypes pointers)"
+        if cannot_cache:
+            msg = ('Cannot cache compiled function "%s" %s'
+                   % (self._funcname, cannot_cache))
+            warnings.warn_explicit(msg, NumbaWarning,
+                                   self._source_path, self._lineno)
+            return False
+        return True
+
+    def _index_key(self, sig, codegen):
+        """
+        Compute index key for the given signature and codegen.
+        It includes a description of the OS and target architecture.
+        """
+        return (sig, codegen.magic_tuple())
+
+    def _data_name(self, number):
+        return self._data_name_pattern.format(number=number)
+
+    def _data_path(self, name):
+        return os.path.join(self._cache_path, name)
+
+    @contextlib.contextmanager
+    def _open_for_write(self, filepath):
+        """
+        Open *filepath* for writing in a race condition-free way
+        (hopefully).
+        """
+        tmpname = '%s.tmp.%d' % (filepath, os.getpid())
+        try:
+            with open(tmpname, "wb") as f:
+                yield f
+            utils.file_replace(tmpname, filepath)
+        except Exception:
+            # In case of error, remove dangling tmp file
+            try:
+                os.unlink(tmpname)
+            except OSError:
+                pass
+            raise
+
+    def _load_index(self):
+        """
+        Load the cache index and return it as a dictionary (possibly
+        empty if cache is empty or obsolete).
+        """
+        try:
+            with open(self._index_path, "rb") as f:
+                version = pickle.load(f)
+                data = f.read()
+        except EnvironmentError as e:
+            # Index doesn't exist yet?
+            if e.errno in (errno.ENOENT,):
+                return {}
+            raise
+        if version != self._version:
+            # This is another version.  Avoid trying to unpickling the
+            # rest of the stream, as that may fail.
+            return {}
+        stamp, overloads = pickle.loads(data)
+        if stamp != self._source_stamp:
+            # Cache is not fresh.  Stale data files will be eventually
+            # overwritten, since they are numbered in incrementing order.
+            return {}
+        else:
+            return overloads
+
+    def _load_data(self, name, target_context):
+        with open(self._data_path(name), "rb") as f:
+            data = f.read()
+        tup = pickle.loads(data)
+        return compiler.CompileResult._rebuild(target_context, *tup)
+
+    def _save_index(self, overloads):
+        data = self._source_stamp, overloads
+        data = self._dump(data)
+        with self._open_for_write(self._index_path) as f:
+            pickle.dump(self._version, f, protocol=-1)
+            f.write(data)
+
+    def _save_data(self, name, cres):
+        data = cres._reduce()
+        data = self._dump(data)
+        with self._open_for_write(self._data_path(name)) as f:
+            f.write(data)
+
+    def _dump(self, obj):
+        return pickle.dumps(obj, protocol=-1)
