@@ -290,7 +290,9 @@ def _lower_array_expr(lowerer, expr):
     for expr_var in expr_var_list:
         expr_var_name = expr_var.name
         expr_var_new_name = expr_var_name.replace("$", "_").replace(".", "_")
-        expr_var_map[expr_var_new_name] = expr_var_name, expr_var
+        # Avoid inserting existing var into the expr_var_map
+        if expr_var_new_name not in expr_var_map:
+            expr_var_map[expr_var_new_name] = expr_var_name, expr_var
         expr_var.name = expr_var_new_name
     expr_filename = expr_var_list[0].loc.filename
     # Parameters are the names internal to the new closure.
@@ -330,11 +332,7 @@ def _lower_array_expr(lowerer, expr):
             inner_sig_args.append(argty)
     inner_sig = outer_sig.return_type.dtype(*inner_sig_args)
 
-    _locals = dict((name, value)
-                   for name, value in namespace.items()
-                   if name.startswith("__ufunc_or_dufunc_"))
-    cres = context.compile_only_no_cache(builder, impl, inner_sig,
-                                         locals=_locals)
+    cres = context.compile_only_no_cache(builder, impl, inner_sig)
 
     class ExprKernel(npyimpl._Kernel):
         def generate(self, *args):
