@@ -15,7 +15,6 @@ from numba import numpy_support
 from numba import types, utils, cgutils, lowering, _helperlib
 
 
-
 class _Registry(object):
 
     def __init__(self):
@@ -82,6 +81,7 @@ class NativeValue(object):
 class EnvironmentManager(object):
 
     def __init__(self, pyapi, env, env_body, env_ptr):
+        assert isinstance(env, lowering.Environment)
         self.pyapi = pyapi
         self.env = env
         self.env_body = env_body
@@ -163,7 +163,6 @@ class PythonAPI(object):
                                                        RuntimeError,
                                                        ("missing Environment",))
 
-
     # ------ Python API -----
 
     #
@@ -743,6 +742,23 @@ class PythonAPI(object):
         elif opstr == 'is not':
             bitflag = self.builder.icmp(lc.ICMP_NE, lhs, rhs)
             return self.from_native_value(bitflag, types.boolean)
+        elif opstr == 'in':
+            fnty = Type.function(Type.int(), [self.pyobj, self.pyobj])
+            fn = self._get_function(fnty, name="PySequence_Contains")
+            status = self.builder.call(fn, (rhs, lhs))
+            negone = self.context.get_constant(types.int32, -1)
+            is_good = self.builder.icmp(lc.ICMP_NE, status, negone)
+            # Stack allocate output and initialize to Null
+            outptr = cgutils.alloca_once_value(self.builder,
+                                               Constant.null(self.pyobj))
+            # If PySequence_Contains returns non-error value
+            with cgutils.if_likely(self.builder, is_good):
+                # Store the status as a boolean object
+                truncated = self.builder.trunc(status, Type.int(1))
+                self.builder.store(self.bool_from_bool(truncated),
+                                   outptr)
+
+            return self.builder.load(outptr)
         else:
             raise NotImplementedError("Unknown operator {op!r}".format(
                 op=opstr))
@@ -1041,8 +1057,6 @@ class PythonAPI(object):
                                                     "prevented the return of " \
                                                     "optional value"
         out = self.from_native_value(val, typ, env_manager)
-        if self.context.enable_nrt:
-            self.context.nrt_decref(self.builder, typ, val)
         return out
 
     def from_native_value(self, val, typ, env_manager=None):
