@@ -9,16 +9,41 @@ from llvmlite import ir, binding as llvm
 
 
 _word_type = ir.IntType(MACHINE_BITS)
+_pointer_type = ir.PointerType(ir.IntType(8))
+
+_meminfo_struct_type = ir.LiteralStructType([
+    _word_type,     # size_t refct
+    _pointer_type,  # dtor_function dtor
+    _pointer_type,  # void *dtor_info
+    _pointer_type,  # void *data
+    _word_type,     # size_t size
+    ])
 
 
-def _define_incref(module, atomic_incr):
+incref_decref_ty = ir.FunctionType(ir.VoidType(), [_pointer_type])
+meminfo_data_ty = ir.FunctionType(_pointer_type, [_pointer_type])
+
+
+def _define_nrt_meminfo_data(module):
+    """
+    Implement NRT_MemInfo_data in the module.  This allows inlined lookup
+    of the data pointer.
+    """
+    fn = module.get_or_insert_function(meminfo_data_ty,
+                                       name="NRT_MemInfo_data")
+    builder = ir.IRBuilder(fn.append_basic_block())
+    [ptr] = fn.args
+    struct_ptr = builder.bitcast(ptr, _meminfo_struct_type.as_pointer())
+    data_ptr = builder.load(cgutils.gep(builder, struct_ptr, 0, 3))
+    builder.ret(data_ptr)
+
+
+def _define_nrt_incref(module, atomic_incr):
     """
     Implement NRT_incref in the module
     """
-    if "NRT_incref" not in module.globals:
-        return
-    fn_incref = module.get_global_variable_named("NRT_incref")
-    fn_incref.linkage = 'linkonce_odr'
+    fn_incref = module.get_or_insert_function(incref_decref_ty,
+                                              name="NRT_incref")
     builder = ir.IRBuilder(fn_incref.append_basic_block())
     [ptr] = fn_incref.args
     is_null = builder.icmp_unsigned("==", ptr, cgutils.get_null_value(ptr.type))
@@ -28,17 +53,14 @@ def _define_incref(module, atomic_incr):
     builder.ret_void()
 
 
-def _define_decref(module, atomic_decr):
+def _define_nrt_decref(module, atomic_decr):
     """
     Implement NRT_decref in the module
     """
-    if "NRT_decref" not in module.globals:
-        return
-    fn_decref = module.get_global_variable_named("NRT_decref")
-    fn_decref.linkage = 'linkonce_odr'
-    calldtor = module.add_function(ir.FunctionType(ir.VoidType(),
-        [ir.IntType(8).as_pointer(), ir.IntType(32)]),
-        name="NRT_MemInfo_call_dtor")
+    fn_decref = module.get_or_insert_function(incref_decref_ty,
+                                              name="NRT_decref")
+    calldtor = module.add_function(ir.FunctionType(ir.VoidType(), [_pointer_type]),
+                                   name="NRT_MemInfo_call_dtor")
 
     builder = ir.IRBuilder(fn_decref.append_basic_block())
     [ptr] = fn_decref.args
@@ -51,25 +73,13 @@ def _define_decref(module, atomic_decr):
     refct_eq_0 = builder.icmp_unsigned("==", newrefct,
                                        ir.Constant(newrefct.type, 0))
     with cgutils.if_unlikely(builder, refct_eq_0):
-        do_defer = ir.Constant(ir.IntType(32), 0)
-        builder.call(calldtor, [ptr, do_defer])
+        builder.call(calldtor, [ptr])
     builder.ret_void()
 
 
-def install_atomic_refct(module):
-    """
-    Implement both NRT_incref and NRT_decref in the module
-    """
-    incref = _define_atomic_inc_dec(module, "add", ordering='monotonic')
-    decref = _define_atomic_inc_dec(module, "sub", ordering='monotonic')
-
-    # Set LinkOnce ODR linkage
-    for fn in [incref, decref]:
-        fn.linkage = 'linkonce_odr'
-    del fn
-
-    _define_incref(module, incref)
-    _define_decref(module, decref)
+# Set this to True to measure the overhead of atomic refcounts compared
+# to non-atomic.
+_disable_atomicity = 0
 
 
 def _define_atomic_inc_dec(module, op, ordering):
@@ -84,11 +94,17 @@ def _define_atomic_inc_dec(module, op, ordering):
     bb = fn_atomic.append_basic_block()
     builder = ir.IRBuilder(bb)
     ONE = ir.Constant(_word_type, 1)
-    oldval = builder.atomic_rmw(op, ptr, ONE, ordering=ordering)
-    # Perform the operation on the old value so that we can pretend returning
-    # the "new" value.
-    res = getattr(builder, op)(oldval, ONE)
-    builder.ret(res)
+    if not _disable_atomicity:
+        oldval = builder.atomic_rmw(op, ptr, ONE, ordering=ordering)
+        # Perform the operation on the old value so that we can pretend returning
+        # the "new" value.
+        res = getattr(builder, op)(oldval, ONE)
+        builder.ret(res)
+    else:
+        oldval = builder.load(ptr)
+        newval = getattr(builder, op)(oldval, ONE)
+        builder.store(newval, ptr)
+        builder.ret(oldval)
 
     return fn_atomic
 
@@ -117,28 +133,32 @@ def _define_atomic_cmpxchg(module, ordering):
     builder.store(old, oldptr)
     builder.ret(builder.zext(ok, ftype.return_type))
 
-
-def define_atomic_ops(module):
-    _define_atomic_inc_dec(module, "add", ordering='monotonic')
-    _define_atomic_inc_dec(module, "sub", ordering='monotonic')
-    _define_atomic_cmpxchg(module, ordering='monotonic')
-    return module
+    return fn_cas
 
 
-def compile_atomic_ops(ctx):
+def compile_nrt_functions(ctx):
+    """
+    Compile all LLVM NRT functions and return a library containing them.
+    The library is created using the given target context.
+    """
     codegen = ctx.jit_codegen()
     library = codegen.create_library("nrt")
 
     # Implement LLVM module with atomic ops
-    ir_mod = define_atomic_ops(library.create_ir_module("atomicops"))
+    ir_mod = library.create_ir_module("nrt_module")
+
+    atomic_inc = _define_atomic_inc_dec(ir_mod, "add", ordering='monotonic')
+    atomic_dec = _define_atomic_inc_dec(ir_mod, "sub", ordering='monotonic')
+    _define_atomic_cmpxchg(ir_mod, ordering='monotonic')
+
+    _define_nrt_meminfo_data(ir_mod)
+    _define_nrt_incref(ir_mod, atomic_inc)
+    _define_nrt_decref(ir_mod, atomic_dec)
+
     library.add_ir_module(ir_mod)
     library.finalize()
 
-    atomic_inc = library.get_pointer_to_function("nrt_atomic_add")
-    atomic_dec = library.get_pointer_to_function("nrt_atomic_sub")
-    atomic_cas = library.get_pointer_to_function("nrt_atomic_cas")
-
-    return library, atomic_inc, atomic_dec, atomic_cas
+    return library
 
 
 _regex_incref = re.compile(r'call void @NRT_incref\((.*)\)')
@@ -205,7 +225,8 @@ def remove_redundant_nrt_refct(ll_module):
             increfs = incref_map[val]
             decrefs = decref_map[val]
             # Mark the incref/decref pairs from the tail for removal
-            for _ in range(min(len(increfs), len(decrefs))):
+            ref_pair_ct = min(len(increfs), len(decrefs))
+            for _ in range(ref_pair_ct):
                 to_remove.add(increfs.pop())
                 to_remove.add(decrefs.popleft())
 
@@ -216,4 +237,3 @@ def remove_redundant_nrt_refct(ll_module):
 
     # Regenerate the LLVM module
     return llvm.parse_assembly(newll)
-
