@@ -1,6 +1,5 @@
 from __future__ import print_function, absolute_import, division
 
-import contextlib
 import sys
 import numpy as np
 import threading
@@ -10,6 +9,7 @@ from numba import unittest_support as unittest
 from numba import njit
 from numba import utils
 from numba.numpy_support import version as numpy_version
+from .support import MemoryLeakMixin
 
 
 nrtjit = njit(_nrt=True, nogil=True)
@@ -32,7 +32,15 @@ class BaseTest(unittest.TestCase):
                 np.testing.assert_allclose(expected, ret)
 
 
-class TestDynArray(unittest.TestCase):
+
+class NrtRefCtTest(MemoryLeakMixin):
+    def assert_array_nrt_refct(self, arr, expect):
+        self.assertEqual(arr.base.refcount, expect)
+
+
+
+class TestDynArray(NrtRefCtTest, unittest.TestCase):
+
     def test_empty_0d(self):
         @nrtjit
         def foo():
@@ -41,6 +49,7 @@ class TestDynArray(unittest.TestCase):
             return arr
 
         arr = foo()
+        self.assert_array_nrt_refct(arr, 1)
         np.testing.assert_equal(42, arr)
         self.assertEqual(arr.size, 1)
         self.assertEqual(arr.shape, ())
@@ -61,6 +70,7 @@ class TestDynArray(unittest.TestCase):
 
         n = 3
         arr = foo(n)
+        self.assert_array_nrt_refct(arr, 1)
         np.testing.assert_equal(np.arange(n), arr)
         self.assertEqual(arr.size, n)
         self.assertEqual(arr.shape, (n,))
@@ -84,6 +94,7 @@ class TestDynArray(unittest.TestCase):
         n = 3
         expected_arr = pyfunc(m, n)
         got_arr = cfunc(m, n)
+        self.assert_array_nrt_refct(got_arr, 1)
         np.testing.assert_equal(expected_arr, got_arr)
 
         self.assertEqual(expected_arr.size, got_arr.size)
@@ -108,6 +119,7 @@ class TestDynArray(unittest.TestCase):
         p = 2
         expected_arr = pyfunc(m, n, p)
         got_arr = cfunc(m, n, p)
+        self.assert_array_nrt_refct(got_arr, 1)
         np.testing.assert_equal(expected_arr, got_arr)
 
         self.assertEqual(expected_arr.size, got_arr.size)
@@ -131,6 +143,7 @@ class TestDynArray(unittest.TestCase):
         p = 2
         expected_arr = pyfunc(m, n, p)
         got_arr = cfunc(m, n, p)
+        self.assert_array_nrt_refct(got_arr, 1)
         np.testing.assert_equal(expected_arr, got_arr)
 
         self.assertEqual(expected_arr.size, got_arr.size)
@@ -172,6 +185,7 @@ class TestDynArray(unittest.TestCase):
 
         cfunc = nrtjit(return_external_array)
         out = cfunc()
+        self.assertIsNone(out.base)
 
         yy = y[2:]
         np.testing.assert_equal(yy, out)
@@ -231,6 +245,8 @@ class TestDynArray(unittest.TestCase):
         np.testing.assert_equal(pyfunc(arr_a, arr_b),
                                 cfunc(arr_a, arr_b))
 
+        self.assert_array_nrt_refct(cfunc(arr_a, arr_b), 1)
+
         # 2D case
         arr_a = np.random.random(10).reshape(2, 5)
         arr_b = np.random.random(10).reshape(2, 5)
@@ -238,6 +254,8 @@ class TestDynArray(unittest.TestCase):
         np.testing.assert_equal(pyfunc(arr_a, arr_b),
                                 cfunc(arr_a, arr_b))
 
+        self.assert_array_nrt_refct(cfunc(arr_a, arr_b), 1)
+
         # 3D case
         arr_a = np.random.random(70).reshape(2, 5, 7)
         arr_b = np.random.random(70).reshape(2, 5, 7)
@@ -245,6 +263,8 @@ class TestDynArray(unittest.TestCase):
         np.testing.assert_equal(pyfunc(arr_a, arr_b),
                                 cfunc(arr_a, arr_b))
 
+        self.assert_array_nrt_refct(cfunc(arr_a, arr_b), 1)
+
     def test_allocation_mt(self):
         """
         This test exercises the array allocation in multithreaded usecase.
@@ -390,7 +410,11 @@ class TestDynArray(unittest.TestCase):
         t = 100
 
         initrefct = sys.getrefcount(x), sys.getrefcount(y)
-        np.testing.assert_equal(pyfunc(x, y, t), cfunc(x, y, t))
+        expect, got = pyfunc(x, y, t), cfunc(x, y, t)
+        self.assertIsNone(got[0].base)
+        self.assertIsNone(got[1].base)
+        np.testing.assert_equal(expect, got)
+        del expect, got
         self.assertEqual(initrefct, (sys.getrefcount(x), sys.getrefcount(y)))
 
     def test_return_tuple_of_array(self):
@@ -459,12 +483,13 @@ class TestDynArray(unittest.TestCase):
         self.assertEqual(old_refct, sys.getrefcount(arr))
 
 
-class ConstructorBaseTest(object):
+class ConstructorBaseTest(NrtRefCtTest):
 
     def check_0d(self, pyfunc):
         cfunc = nrtjit(pyfunc)
         expected = pyfunc()
         ret = cfunc()
+        self.assert_array_nrt_refct(ret, 1)
         self.assertEqual(ret.size, expected.size)
         self.assertEqual(ret.shape, expected.shape)
         self.assertEqual(ret.dtype, expected.dtype)
@@ -481,6 +506,7 @@ class ConstructorBaseTest(object):
         n = 3
         expected = pyfunc(n)
         ret = cfunc(n)
+        self.assert_array_nrt_refct(ret, 1)
         self.assertEqual(ret.size, expected.size)
         self.assertEqual(ret.shape, expected.shape)
         self.assertEqual(ret.dtype, expected.dtype)
@@ -501,6 +527,7 @@ class ConstructorBaseTest(object):
         m, n = 2, 3
         expected = pyfunc(m, n)
         ret = cfunc(m, n)
+        self.assert_array_nrt_refct(ret, 1)
         self.assertEqual(ret.size, expected.size)
         self.assertEqual(ret.shape, expected.shape)
         self.assertEqual(ret.dtype, expected.dtype)
@@ -520,6 +547,7 @@ class ConstructorBaseTest(object):
 class TestNdZeros(ConstructorBaseTest, unittest.TestCase):
 
     def setUp(self):
+        super(TestNdZeros, self).setUp()
         self.pyfunc = np.zeros
 
     def check_result_value(self, ret, expected):
@@ -567,6 +595,7 @@ class TestNdZeros(ConstructorBaseTest, unittest.TestCase):
 class TestNdOnes(TestNdZeros):
 
     def setUp(self):
+        super(TestNdOnes, self).setUp()
         self.pyfunc = np.ones
 
 
@@ -643,6 +672,7 @@ class ConstructorLikeBaseTest(object):
 class TestNdEmptyLike(ConstructorLikeBaseTest, unittest.TestCase):
 
     def setUp(self):
+        super(TestNdEmptyLike, self).setUp()
         self.pyfunc = np.empty_like
 
     def check_result_value(self, ret, expected):
@@ -691,6 +721,7 @@ class TestNdEmptyLike(ConstructorLikeBaseTest, unittest.TestCase):
 class TestNdZerosLike(TestNdEmptyLike):
 
     def setUp(self):
+        super(TestNdZerosLike, self).setUp()
         self.pyfunc = np.zeros_like
 
     def check_result_value(self, ret, expected):
@@ -711,6 +742,7 @@ class TestNdZerosLike(TestNdEmptyLike):
 class TestNdOnesLike(TestNdZerosLike):
 
     def setUp(self):
+        super(TestNdOnesLike, self).setUp()
         self.pyfunc = np.ones_like
         self.expected_value = 1
 
