@@ -32,6 +32,11 @@ class Integer(Number):
         self.bitwidth = bitwidth
         self.signed = self.name.startswith('int')
 
+    @classmethod
+    def from_bitwidth(cls, bitwidth, signed=True):
+        name = ('int%d' if signed else 'uint%d') % bitwidth
+        return globals()[name]
+
     def cast_python_value(self, value):
         return getattr(numpy, self.name)(value)
 
@@ -99,6 +104,13 @@ class _NPDatetimeBase(Type):
         # also greater...).
         return self.unit_code < other.unit_code
 
+    def cast_python_value(self, value):
+        cls = getattr(numpy, self.type_name)
+        if self.unit:
+            return cls(value, self.unit)
+        else:
+            return cls(value)
+
 
 @utils.total_ordering
 class NPTimedelta(_NPDatetimeBase):
@@ -116,6 +128,16 @@ class Phantom(Dummy):
     """
 
 
+class Undefined(Dummy):
+    """
+    A type that is left imprecise.  This is used as a temporaray placeholder
+    during type inference in the hope that the type can be later refined.
+    """
+
+    def is_precise(self):
+        return False
+
+
 class Opaque(Dummy):
     """
     A type that is a opaque pointer.
@@ -127,6 +149,9 @@ class PyObject(Dummy):
     A generic CPython object.
     """
 
+    def is_precise(self):
+        return False
+
 
 class RawPointer(Dummy):
     """
@@ -136,6 +161,8 @@ class RawPointer(Dummy):
 
 class Kind(Type):
     def __init__(self, of):
+        if not isinstance(of, type) or not issubclass(of, Type):
+            raise TypeError("expected a Type subclass, got %r" % (of,))
         self.of = of
         super(Kind, self).__init__("kind(%s)" % of)
 
@@ -194,20 +221,53 @@ class Function(Callable, Opaque):
     def get_call_type(self, context, args, kws):
         return self.template(context).apply(args, kws)
 
+    def get_call_signatures(self):
+        sigs = getattr(self.template, 'cases', [])
+        return sigs, hasattr(self.template, 'generic')
+
+
+class NamedTupleClass(Callable, Opaque):
+    """
+    Type class for namedtuple classes.
+    """
+
+    def __init__(self, instance_class):
+        self.instance_class = instance_class
+        name = "class(%s)" % (instance_class)
+        super(NamedTupleClass, self).__init__(name, param=True)
+
+    def get_call_type(self, context, args, kws):
+        # Overriden by the __call__ constructor resolution in typing.collections
+        return None
+
+    def get_call_signatures(self):
+        return (), True
+
+    @property
+    def key(self):
+        return self.instance_class
+
 
 class NumberClass(Callable, DTypeSpec, Opaque):
     """
     Type class for number classes (e.g. "np.float64").
     """
 
-    def __init__(self, instance_type, template):
+    def __init__(self, instance_type):
         self.instance_type = instance_type
-        self.template = template
-        name = "type(%s)" % (instance_type,)
-        super(NumberClass, self).__init__(name)
+        name = "class(%s)" % (instance_type,)
+        super(NumberClass, self).__init__(name, param=True)
 
     def get_call_type(self, context, args, kws):
-        return self.template(context).apply(args, kws)
+        # Overriden by the __call__ constructor resolution in typing.builtins
+        return None
+
+    def get_call_signatures(self):
+        return (), True
+
+    @property
+    def key(self):
+        return self.instance_type
 
     @property
     def dtype(self):
@@ -225,6 +285,10 @@ class DType(DTypeSpec, Opaque):
         name = "dtype(%s)" % (dtype,)
         super(DTypeSpec, self).__init__(name)
 
+    @property
+    def key(self):
+        return self.dtype
+
     @property
     def dtype(self):
         return self._dtype
@@ -270,6 +334,10 @@ class Dispatcher(WeakType, Callable, Dummy):
         sig.pysig = self.pysig
         return sig
 
+    def get_call_signatures(self):
+        sigs = self.overloaded.nopython_signatures
+        return sigs, True
+
     @property
     def overloaded(self):
         """
@@ -370,18 +438,13 @@ class BoundFunction(Function):
                       dict(this=this))
         super(BoundFunction, self).__init__(newcls)
 
-    @property
-    def key(self):
-        return self.template.key, self.this
-
-
-class Method(Function):
-    def __init__(self, template, this):
-        self.this = this
-        # Create a derived template with an attribute *this*
-        newcls = type(template.__name__ + '.' + str(this), (template,),
-                      dict(this=this))
-        super(Method, self).__init__(newcls)
+    def unify(self, typingctx, other):
+        if (isinstance(other, BoundFunction) and
+            self.template.key == other.template.key):
+            this = typingctx.unify_pairs(self.this, other.this)
+            if this != pyobject:
+                # XXX is it right that both template instances are distinct?
+                return BoundFunction(self.template, this)
 
     @property
     def key(self):
@@ -403,6 +466,13 @@ class Pair(Type):
     def key(self):
         return self.first_type, self.second_type
 
+    def unify(self, typingctx, other):
+        if isinstance(other, Pair):
+            first = typingctx.unify_pairs(self.first_type, other.first_type)
+            second = typingctx.unify_pairs(self.second_type, other.second_type)
+            if first != pyobject and second != pyobject:
+                return Pair(first, second)
+
 
 class SimpleIterableType(IterableType):
 
@@ -455,7 +525,7 @@ class Generator(SimpleIteratorType):
         return self.gen_func, self.arg_types, self.yield_type, self.has_finalizer
 
 
-class NumpyFlatType(SimpleIteratorType):
+class NumpyFlatType(SimpleIteratorType, Sequence):
     """
     Type class for `ndarray.flat()` objects.
     """
@@ -463,6 +533,7 @@ class NumpyFlatType(SimpleIteratorType):
     def __init__(self, arrty):
         self.array_type = arrty
         yield_type = arrty.dtype
+        self.dtype = yield_type
         name = "array.flat({arrayty})".format(arrayty=arrty)
         super(NumpyFlatType, self).__init__(name, yield_type)
 
@@ -566,20 +637,29 @@ class UnicodeCharSeq(Type):
 
 
 class Record(Type):
+    """
+    A Numpy structured scalar.  *descr* is the string representation
+    of the Numpy dtype; *fields* of mapping of field names to
+    (type, offset) tuples; *size* the bytesize of a record;
+    *aligned* whether the fields are aligned; *dtype* the Numpy dtype
+    instance.
+    """
     mutable = True
 
-    def __init__(self, id, fields, size, aligned, dtype):
-        self.id = id
+    def __init__(self, descr, fields, size, aligned, dtype):
+        self.descr = descr
         self.fields = fields.copy()
         self.size = size
         self.aligned = aligned
         self.dtype = dtype
-        name = 'Record(%s)' % id
+        name = 'Record(%s)' % descr
         super(Record, self).__init__(name)
 
     @property
     def key(self):
-        return (self.dtype, self.size, self.aligned)
+        # Numpy dtype equality doesn't always succeed, use the descr instead
+        # (https://github.com/numpy/numpy/issues/5715)
+        return (self.descr, self.size, self.aligned)
 
     def __len__(self):
         return len(self.fields)
@@ -638,11 +718,10 @@ class Buffer(IterableType):
                 type_name = "readonly %s" % type_name
             name = "%s(%s, %sd, %s)" % (type_name, dtype, ndim, layout)
         super(Buffer, self).__init__(name, param=True)
-        self._iterator_type = ArrayIterator(self)
 
     @property
     def iterator_type(self):
-        return self._iterator_type
+        return ArrayIterator(self)
 
     def copy(self, dtype=None, ndim=None, layout=None):
         if dtype is None:
@@ -832,19 +911,39 @@ class BaseTuple(Type):
     The base class for all tuple types (with a known size).
     """
 
+    @classmethod
+    def from_types(cls, tys, pyclass=None):
+        """
+        Instantiate the right tuple type for the given element types.
+        """
+        homogenous = False
+        if tys:
+            first = tys[0]
+            for ty in tys[1:]:
+                if ty != first:
+                    break
+            else:
+                homogenous = True
 
-class UniTuple(IterableType, BaseTuple):
+        if pyclass is not None and pyclass is not tuple:
+            # A subclass => is it a namedtuple?
+            assert issubclass(pyclass, tuple)
+            if hasattr(pyclass, "_asdict"):
+                if homogenous:
+                    return NamedUniTuple(first, len(tys), pyclass)
+                else:
+                    return NamedTuple(tys, pyclass)
+        if homogenous:
+            return UniTuple(first, len(tys))
+        else:
+            return Tuple(tys)
 
-    def __init__(self, dtype, count):
-        self.dtype = dtype
-        self.count = count
-        name = "(%s x %d)" % (dtype, count)
-        super(UniTuple, self).__init__(name, param=True)
-        self._iterator_type = UniTupleIter(self)
+
+class _HomogenousTuple(Sequence, BaseTuple):
 
     @property
     def iterator_type(self):
-        return self._iterator_type
+        return UniTupleIter(self)
 
     def getitem(self, ind):
         return self.dtype, intp
@@ -861,14 +960,26 @@ class UniTuple(IterableType, BaseTuple):
     def __len__(self):
         return self.count
 
-    @property
-    def key(self):
-        return self.dtype, self.count
-
     @property
     def types(self):
         return (self.dtype,) * self.count
 
+
+class UniTuple(_HomogenousTuple):
+    """
+    Type class for homogenous tuples.
+    """
+
+    def __init__(self, dtype, count):
+        self.dtype = dtype
+        self.count = count
+        name = "(%s x %d)" % (dtype, count)
+        super(UniTuple, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.dtype, self.count
+
     def unify(self, typingctx, other):
         """
         Unify UniTuples with their dtype
@@ -899,13 +1010,7 @@ class UniTupleIter(SimpleIteratorType):
         return self.unituple
 
 
-class Tuple(BaseTuple):
-
-    def __init__(self, types):
-        self.types = tuple(types)
-        self.count = len(self.types)
-        name = "(%s)" % ', '.join(str(i) for i in self.types)
-        super(Tuple, self).__init__(name, param=True)
+class _HeterogenousTuple(BaseTuple):
 
     def __getitem__(self, i):
         """
@@ -917,13 +1022,22 @@ class Tuple(BaseTuple):
         # Beware: this makes Tuple(()) false-ish
         return len(self.types)
 
+    def __iter__(self):
+        return iter(self.types)
+
+
+class Tuple(_HeterogenousTuple):
+
+    def __init__(self, types):
+        self.types = tuple(types)
+        self.count = len(self.types)
+        name = "(%s)" % ', '.join(str(i) for i in self.types)
+        super(Tuple, self).__init__(name, param=True)
+
     @property
     def key(self):
         return self.types
 
-    def __iter__(self):
-        return iter(self.types)
-
     def unify(self, typingctx, other):
         """
         Unify elements of Tuples/UniTuples
@@ -948,6 +1062,125 @@ class Tuple(BaseTuple):
             return max(kinds)
 
 
+class BaseNamedTuple(BaseTuple):
+    pass
+
+
+class NamedUniTuple(_HomogenousTuple, BaseNamedTuple):
+
+    def __init__(self, dtype, count, cls):
+        self.dtype = dtype
+        self.count = count
+        self.fields = tuple(cls._fields)
+        self.instance_class = cls
+        name = "%s(%s x %d)" % (cls.__name__, dtype, count)
+        super(NamedUniTuple, self).__init__(name, param=True)
+        self._iterator_type = UniTupleIter(self)
+
+    @property
+    def key(self):
+        return self.instance_class, self.dtype, self.count
+
+
+class NamedTuple(_HeterogenousTuple, BaseNamedTuple):
+
+    def __init__(self, types, cls):
+        self.types = tuple(types)
+        self.count = len(self.types)
+        self.fields = tuple(cls._fields)
+        self.instance_class = cls
+        name = "%s(%s)" % (cls.__name__, ', '.join(str(i) for i in self.types))
+        super(NamedTuple, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.instance_class, self.types
+
+
+class List(MutableSequence):
+    """
+    Type class for arbitrary-sized homogenous lists.
+    """
+    mutable = True
+
+    def __init__(self, dtype):
+        self.dtype = dtype
+        name = "list(%s)" % (self.dtype,)
+        super(List, self).__init__(name=name, param=True)
+
+    def unify(self, typingctx, other):
+        if isinstance(other, List):
+            dtype = typingctx.unify_pairs(self.dtype, other.dtype)
+            if dtype != pyobject:
+                return List(dtype=dtype)
+
+    @property
+    def key(self):
+        return self.dtype
+
+    @property
+    def iterator_type(self):
+        return ListIter(self)
+
+    def is_precise(self):
+        return self.dtype.is_precise()
+
+
+class ListIter(SimpleIteratorType):
+    """
+    Type class for list iterators.
+    """
+
+    def __init__(self, list_type):
+        self.list_type = list_type
+        yield_type = list_type.dtype
+        name = 'iter(%s)' % list_type
+        super(ListIter, self).__init__(name, yield_type)
+
+    # XXX This is a common pattern.  Should it be factored out somewhere?
+    def unify(self, typingctx, other):
+        if isinstance(other, ListIter):
+            list_type = typingctx.unify_pairs(self.list_type, other.list_type)
+            if list_type != pyobject:
+                return ListIter(list_type)
+
+    @property
+    def key(self):
+        return self.list_type
+
+
+class ListPayload(Type):
+    """
+    Internal type class for the dynamically-allocated payload of a list.
+    """
+
+    def __init__(self, list_type):
+        self.list_type = list_type
+        name = 'payload(%s)' % list_type
+        super(ListPayload, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.list_type
+
+
+class MemInfoPointer(Type):
+    """
+    Pointer to a Numba "meminfo" (i.e. the information for a managed
+    piece of memory).
+    """
+    mutable = True
+
+    def __init__(self, dtype):
+        self.dtype = dtype
+        name = "memory-managed *%s" % dtype
+        super(MemInfoPointer, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.dtype
+
+
 class CPointer(Type):
     """
     Type class for pointers to other types.
@@ -1055,7 +1288,7 @@ class NoneType(Opaque):
         return Optional(other)
 
 
-class ExceptionType(Callable, Phantom):
+class ExceptionClass(Callable, Phantom):
     """
     The type of exception classes (not instances).
     """
@@ -1064,12 +1297,15 @@ class ExceptionType(Callable, Phantom):
         assert issubclass(exc_class, BaseException)
         name = "%s" % (exc_class.__name__)
         self.exc_class = exc_class
-        super(ExceptionType, self).__init__(name, param=True)
+        super(ExceptionClass, self).__init__(name, param=True)
 
     def get_call_type(self, context, args, kws):
+        return self.get_call_signatures()[0][0]
+
+    def get_call_signatures(self):
         from . import typing
         return_type = ExceptionInstance(self.exc_class)
-        return typing.signature(return_type)
+        return [typing.signature(return_type)], False
 
     @property
     def key(self):
@@ -1097,32 +1333,19 @@ class Slice3Type(Type):
     pass
 
 
-# Utils
-
-def is_int_tuple(x):
-    if isinstance(x, Tuple):
-        return all(i in integer_domain for i in x.types)
-    elif isinstance(x, UniTuple):
-        return x.dtype in integer_domain
-    else:
-        return False
-
-
 # Short names
 
 pyobject = PyObject('pyobject')
 ffi_forced_object = Opaque('ffi_forced_object')
 none = NoneType('none')
 Any = Phantom('any')
+undefined = Undefined('undefined')
 string = Opaque('str')
 
 # No operation is defined on voidptr
 # Can only pass it around
 voidptr = RawPointer('void*')
 
-# For NRT GC
-meminfo_pointer = Opaque("MemInfo*")
-
 boolean = bool_ = Boolean('bool')
 
 byte = uint8 = Integer('uint8')
