@@ -2,10 +2,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, division, absolute_import
 import sys
+
 import numba.testing as testing
-if '-m' in sys.argv:
-    sys.argv.remove('-m')
-    result = testing.multitest()
-else:
+
+if __name__ == "__main__":
     result = testing.test()
-sys.exit(0 if result else 1)
+    sys.exit(0 if result else 1)
